﻿/* ***************************************
 * Logic analizer rev 1 mod 2 2020-12-05
 * add:Dspblack: white display background 
 *     for inverted screen
 * copyright 2020 iwamoto
 * Curie new のフォントに変更　トリガのコントロールを追加
 * ***************************************/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO.Ports;

namespace LogicAnalizer
{
    public partial class Form1 : Form
    {
        //シリアルデータの受信をUIスレッドに通知する
        // delegate function を作成する
        delegate void ReadDataQue();
        string RxStr = "@@@@";        // @からの64文字（6bit）を使用
        int Xpos = 100;               // 画面　Xカーソルの位置
        int Ypos = 100;               // 画面　Yカーソルの位置
        bool isIdle = true;           // Run/Idleが Idleなら真

        public Form1()
        {
            InitializeComponent();       
            UpdateCOMPortList();     // COM ports listを作成する
        }
        /**************************  ポート更新 *************************************
         * private void UpdateCOMPortList()
         * 
         * Function:
         *     the COM ports listbox を定期的にUpDateする
         *     周期はTimer1のプロパティでで定める
        ***************************************************************************/
        private void UpdateCOMPortList()
        {
            int i = 0;
            bool foundDifference = false;
            // COM port数が前回と異なっていれば、
            // COM portの抜き差しが行われたことになる
            if (lstCOMPorts.Items.Count == SerialPort.GetPortNames().Length)
            {
                // 有効な全ての SerialPort COM port 名と
                // lstCOMPorts 内のリストと同じか比較する
                foreach (string s in SerialPort.GetPortNames())
                {
                    //もし、ひとつでも異なればリストをUpdateする
                    if (lstCOMPorts.Items[i++].Equals(s) == false)
                    {
                        foundDifference = true;
                    }
                }
            }
            else
            {
                foundDifference = true;
            }
            //前回と同一なら Return
            if (foundDifference == false) return;
            //異なれば、リストをクリア
            lstCOMPorts.Items.Clear();
            //接続がなければ Return
            if (SerialPort.GetPortNames().Length == 0) return;
            // COM ports listをUpDateする
            foreach (string s in SerialPort.GetPortNames())
            {
                lstCOMPorts.Items.Add(s);
            }
            // listbox の先頭アイテムを選択
            lstCOMPorts.SelectedIndex = 0;
        }

        /************************** タイマ１ *****************************************
         * private void timer1_Tick(object sender, EventArgs e)
         * 
         * Function: serialPort1 が、接続されていないときは、
         *           COM ports listboxの定期確認 用の周期を発生させるr（標準200mS）
       ***************************************************************************/
        private void timer1_Tick(object sender, EventArgs e)
        {
            // COM ports は接続されていない
            //  接続に備え COM portリストをUpDateする
            UpdateCOMPortList();
        }

        /************************** タイマ２ *****************************************
        * private void timer2_Tick(object sender, EventArgs e)
        * 
        * Function: Free Run のトリガ発生周期
        *           （標準1000mS）
      ***************************************************************************/
        private void timer2_Tick(object sender, EventArgs e)
        {
            if (isIdle)   //if (btnRun.Text == "Run")          // ボタンがRun 表示 なら
            {
                timer2.Enabled = false;                // Timer2を止めて、
                return;                                // 復帰
            }
            if (rb3_FreeRun.Checked == false)   // FreeRun　でない  なら
            {
                btnRun.UseVisualStyleBackColor = true; // Cancel を　
                btnRun.Text = "Run";                   // Run に変更
                isIdle = true;                
                timer2.Enabled = false;                // Timer2を止めて、
                return;                                // 復帰
            }
            // ------------------------ FreeRun 実行中なら -------------
            sendCmd();                        // パネルのコマンドを送信
            dspCursorPos();                   // カーソルを再描画
            pb_LED_ON.Visible = true;         // LED 点灯
        }

        /************************** 接続ボタン ***************************************
         * private void btnConnect_Click(object sender, EventArgs e)
         * 
         * Description:
         *     btnConnect がクリックされた時に実行され指定されたCOM portを接続する
         *     さらに、必要な操作関連部品を有効・無効にする
        ***************************************************************************/
        private void btnConnect_Click(object sender, System.EventArgs e)
        {
            // COM portが、ひとつもなければ、何もしない。
            if (lstCOMPorts.Text == "") return;

            // 指定されたCOM portを接続し、操作関連部品を有効・無効にする
            //  COM portの接続時は、エラー回避のために try/catch を使用すること
            //  が重要。 USB virtual COM port が、抜かれたときに例外事項として対応できる
            try
            {
                //timer1は、UpdateCOMPortListでの使用をやめ
                //Port Closeのタイミング発生に備える
                timer1.Enabled = false;

                //COMPortListで指定れている PortName (e.g. - "COM1")を
                //  取得し、COM portを接続する
                serialPort1.PortName = lstCOMPorts.Text;
                serialPort1.Open();

                //操作関連部品の有効・無効を変更する
                btnConnect.Enabled = false;
                lstCOMPorts.Enabled = false;
                btnClose.Enabled = true;
                btnClose.Focus();
                btnRun.Enabled = true;
            }
            catch
            {
                //例外が発生した場合、機器が取はずされたと思われるので 
                //  PortをCloseする
                btnClose_Click(this, null);
            }
        }

        /**************************  切断ボタン *************************************
         * private void btnClose_Click(object sender, EventArgs e)
         * 
         * Description:
         *      切断（Close)ボタンがクリックされたとき実行される以外にも、他のルーチン
         *      からも直接コールされる場合がある。
         *      ここでは、操作関連部品の有効・無効の変更が行われ、実際のCOM Portの
         *      Closeは、200mS後にtimer1_Tickルーチンで実施される
        ***************************************************************************/
        private void btnClose_Click(object sender, EventArgs e)
        {
            //Reset the state of the application objects
            btnRun.UseVisualStyleBackColor = true; // Cancel を　
            btnRun.Text = "Run";                   // Run に変更
            isIdle = true;
            btnRun.Enabled = false;
            btnClose.Enabled = false;
            btnConnect.Enabled = true;
            lstCOMPorts.Enabled = true;
            timer1.Enabled = true;
            timer2.Enabled = false;
            try
            {
                //Dispose the In and Out buffers;
                serialPort1.DiscardInBuffer();
                serialPort1.DiscardOutBuffer();

                //Close the COM port
                serialPort1.Close();
            }
            //If there was an exeception then there isn't much we can
            //  do.  The port is no longer available.
            catch { }
        }

        /*************************  受信イベント ************************************
         * private void serialPort1_DataReceived(  object sender,
         *                                        SerialDataReceivedEventArgs e)
         * Description:
         *      COM port でデータを受信したとき実行される
         *      このイベントはセカンドスレッドで発生するため textbox
         *      のあるUIスレッドに伝達するために、デリゲートを利用する 
        ****************************************************************************/
        private void serialPort1_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            ReadDataQue rt = new ReadDataQue(ReadText);
            // delegate をInvoke（実施）することで異なった
            //  スレッドの機能に伝送している
            //Invoke(rt, new object[] { });
            Invoke(rt);
        }

        /*************************  データ受信 *************************************
         *  private void ReadText()
         *  Description:
         *  [Run]ボタンの時は、    ：isIdle = true [Run]ボタン背景が白の時は、
         *      何もせずに復帰
         *  [Cancel]ボタンの時は、  ：isIdle = false [Run]ボタン背景が橙の時は、
         *      受信バッファから一行読み取り、前後の空文字を削除する
         *      データの終わりは「＃＃」で示す                
        ***************************************************************************/
        private void ReadText()
        {
            // [Run]ボタンの時は、何もせずに復帰
            if (isIdle) return;   //if (btnRun.Text == "Run") return;

            // [Cancel][Stop]ボタンの時は、
            string RxText = "";
            try
            {
                RxStr = "";
                while (true)
                {
                    RxText = serialPort1.ReadLine();        // 一行読み取り
                    RxText = RxText.Trim('\r', '\n', '\0'); // 前後の空文字
                    if (RxText == "##") break;              // データの終わり
                    if (RxText == "!!") break;              // トリガの中止
                    //  単に、 text を追記している.
                    RxStr += RxText;
                }
            }
            //TimeOut例外が発生した場合、
            catch
            {
                ShowMsg("Time Out.\r\n");
            }
            //メッセージ欄を非表示に、
            ShowMsg("");
            if (rb3_FreeRun.Checked == false)          // FreeRun 以外 ------------
            {
                btnRun.UseVisualStyleBackColor = true; //    Cancel を　
                btnRun.Text = "Run";                   //    Run に変更
                isIdle = true;
            }
            else                                       // FreeRun  ------------
            {
                pb_LED_ON.Visible = false;             //    LED を消灯
            }
            if (RxText == "##") dataDraw();            // 正常受信なので描画
            if (RxText == "!!") screenBack();          // エラーなので画面クリア
        }

        /*************************  dataDraw ****************************************
        *  private void dataDraw
        *  
        * Description:
        *    目盛を描く
        *    データ線を描く
        *    リソースを解放する
      ***************************************************************************/
        private void dataDraw()
        { 
            int i;
            // -----------------------------------------------------------------
            //描画先とするImageオブジェクトを作成する
            //ImageオブジェクトのGraphicsオブジェクトを作成する
            // -----------------------------------------------------------------
            Bitmap canvas = new Bitmap(pictureBox1.Width, pictureBox1.Height);
            Graphics g = Graphics.FromImage(canvas);

            // -----------------------------------------------------------------
            // 目盛を描く  canvasの幅と高さを基準に10pixの間隔
            //             副目盛は、上下に5pix高、主目盛は高さ一杯
            // -----------------------------------------------------------------
            for (i = 0; i < (canvas.Width / 10); i++)
            {
                if (i % 5 == 0)
                {   // 主目盛は、上下の貫通線
                    g.DrawLine(Pens.DarkGray, 10 * i, 0, 10 * i, canvas.Height);
                }
                else
                {   // 副目盛は、枠の上下に高さ5pixの線
                    g.DrawLine(Pens.DarkGray, 10 * i, 0, 10 * i, 5);
                    g.DrawLine(Pens.DarkGray, 10 * i, canvas.Height - 5, 10 * i, canvas.Height);
                }
            }

            // -----------------------------------------------------------------
            // トリガ線を描く  トリガ位置に赤線
            // -----------------------------------------------------------------
            int Tpos = 0;
            if (rb0_Start.Checked) Tpos = 100;
            else if (rb4_Center.Checked) Tpos = 550;
            else if (rb5_End.Checked) Tpos = 1000;
            if (Tpos != 0) g.DrawLine(Pens.Red, Tpos, 0, Tpos, canvas.Height);
           
            // -----------------------------------------------------------------
            // データ線を描く  
            //             
            // Data      bit0はCh1   bit1はCh2
            //           bit2はCh3   bit3はCh4
            // -----------------------------------------------------------------
            int smpPoint;  // 描いているデータ位置
            int dataNow;   // 描いているデータ位置
            int dataNext;  // 描いているデータ位置
            int diffData;  // 描いていると次に描くデータの変位
            int ch;        // 描いているチャネル
            int testBit;   // 上記のチャネルbit位置
            int HiPos;     // 上記のチャネルHiレベル描画位置
            int LoPos;     // 上記のチャネルLowレベル描画位置

            // GND用太ペンの作成 color chanfed by DspBack
            Pen boldPen = new Pen(Color.Lime, 2);
            Pen nomlPen = new Pen(Color.Lime, 1);
            if (cbDspBlack.Checked)
            {
                boldPen.Color = Color.Magenta;
                nomlPen.Color = Color.Magenta;
            }
            // 各サンプルデータの数だけ繰り返す ----------------------------
            for (smpPoint = 0; smpPoint < (RxStr.Length)-1; smpPoint++)
            {
                // サンプルデータ毎の初期設定
                //    testBitをCh1を示すBit位置に戻す
                //    文字列から必要データを抜き出し、bit値に直す
                //    次のサンプルとのLogic変位を計算
                testBit = 1;
                dataNow  = (Convert.ToInt32(RxStr[smpPoint]))   & 0xF;
                dataNext = (Convert.ToInt32(RxStr[smpPoint+1])) & 0xF;
                diffData = dataNow ^ dataNext;

                // 各チャネルの描画  ４Ch分繰り返す ---------------------
                for (ch = 0; ch < 4; ch++)
                {
                    HiPos = ch * 30 + 10;   // Logic Hi の画描位置を計算
                    LoPos = HiPos + 20;     // Logic Lowの画描位置を計算
                    // 現在のレベル位置に横線を引く
                    if ((dataNow & testBit) == testBit)
                        g.DrawLine(nomlPen, smpPoint, HiPos, smpPoint + 1, HiPos);                    
                    else
                        g.DrawLine(boldPen, smpPoint, LoPos, smpPoint + 1, LoPos);
                    // 次のデータのレベルが変位していれば、縦線を引く
                    if ((diffData & testBit) == testBit)
                        g.DrawLine(nomlPen, smpPoint+1, LoPos, smpPoint + 1, HiPos);
                    testBit <<= 1;      // 次のチャンネル準備
                }                       // 次のチャンネル
            }                           // 次のサンプル  
            //リソースを解放する---------------------------------------------------
            boldPen.Dispose();
            nomlPen.Dispose();
            g.Dispose();                //
            pictureBox1.Image = canvas; //PictureBox1に表示する
        }

        /**************************  Run/Cancelボタン *******************************
         *  private void btnRun_Click(object sender, EventArgs e)
         *  
         *  Description:
         *  [Run]ボタンは操作パネルの情報を組み合わせて、
         *      コマンドをPICに送信
         *      ボタン名を[Cancel]に変更する
         *  [Cancel]ボタンが押されればトリガ待ちを中止するために、
         *      強制FreeRunコマンド「！」をPICに送信                
        ***************************************************************************/
        private void btnRun_Click(object sender, EventArgs e)
        {
            // ------------------------------------------------------------------------
            // コマンド設定の確認 
            // -----------------------------------------------------------------------
            if (rb3_FreeRun.Checked == false)
            {                             // FreeRun 以外のトリガ ---------------------
                if(isIdle) //if (btnRun.Text == "Run")     // ボタンの表示が「Run」なら
                {                             // 操作待ちなので
                    sendCmd();                // パネルのコマンドを送信
                    dspCursorPos();
                }                             // 
                else                          // ボタンの表示が「Cancel」で
                {                             // トリガが掛るのを待っているなら
                    freeCmd();                // 強制FreeRunコマンド送信
                }
            }
            else 
            {                             // FreeRun の場合 ---------------------
                if (isIdle) //if (btnRun.Text == "Run")     // ボタンの表示が「Run」なら
                {                                 // 操作待ちなので
                    sendCmd();                    // パネルのコマンドを送信
                    dspCursorPos();               // カーソルを再描画
                    timer2.Enabled = true;        // タイマー2をON
                }                                 //   （1秒ごとにFreeRun)
                else
                {                             // ボタンの表示が「Stop」でなら
                    timer2.Enabled = false;       // timer2 を止め
                    pb_LED_ON.Visible = false;    // LED OFF
                    btnRun.Text = "Run";          // ボタンの表示をRun に戻す
                    btnRun.UseVisualStyleBackColor = true;
                    isIdle = true;
                }
            }
        }

        /**************************  sendCmd()関数 *********************************
         * private void sendCmd()
         * 
         * Description:
         * 操作パネルの情報を組み合わせて、コマンドをPICに送信、PICから返信を待ち
         * エラー「！」なら、
         *     コマンドを再送２度失敗したら、ダイアログでえらーを通知
         * 正常返信「＃」なら、
         *     ボタン名をCancel、色をゴールドに変えトリガ結果を待つ
        ***************************************************************************/
        private void sendCmd()
        {
            // ------------------------------------------------------------------------
            // コマンド設定の確認 
            // ------------------------------------------------------------------------
            string errMsg = "";
            ShowMsg("");
            if (rb3_FreeRun.Checked == false)  // FreeRunの時は制約なし
            {
                int edgeCnt = 0;
                if (cbCh1Trig.SelectedIndex > 2) edgeCnt++;
                if (cbCh2Trig.SelectedIndex > 2) edgeCnt++;
                if (cbCh3Trig.SelectedIndex > 2) edgeCnt++;
                if (edgeCnt != 1)
                {
                    errMsg = "チャンネルのひとつに「?」または「?」を指定してください";
                    ShowMsg(errMsg);
                    return;
                }
            }
            // ------------------------------------------------------------------------
            // コマンドワードの作成 
            // ------------------------------------------------------------------------
            string TxData = "#";
            TxData += cbCh1Trig.SelectedIndex.ToString();    // Ch1のトリガ条件セット
            TxData += cbCh2Trig.SelectedIndex.ToString();    // Ch2
            TxData += cbCh3Trig.SelectedIndex.ToString();    // Ch3
            TxData += "#";            // SampleRateが選ばれてなければ、50usとする
            if (cbSampleRate.SelectedIndex == -1) cbSampleRate.SelectedIndex = 7;
            TxData += cbSampleRate.SelectedIndex.ToString(); // SampleRateをセット
            if (rb0_Start.Checked) TxData += "0";     // トリガ位置をセット
            else if (rb1_Dly1Win.Checked) TxData += "1";     // 
            else if (rb2_Dly2Win.Checked) TxData += "2";     // 
            else if (rb3_FreeRun.Checked) TxData += "3";     // 
            else if (rb4_Center.Checked) TxData += "4";     // 
            else if (rb5_End.Checked) TxData += "5";     // 

            //tbMsg.Text = TxData; //<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

            // ------------------------------------------------------------------------
            // コマンドの送信と返答の受信 
            // ------------------------------------------------------------------------
            try
            {   //コマンドをシリアルポートから送信する
                int TxCnt = 0;                               // 送信トライ回数
                serialPort1.DiscardInBuffer();               // すでに受信しているデータの廃棄
                while (true)
                {
                    serialPort1.Write(TxData + "\r\n");      // 送信する
                    // 返答を待つ 「＃」ならPICで正常処理されたのでループを抜ける
                    if (serialPort1.ReadLine().StartsWith("#")) break;
                    if (TxCnt++ > 0)                        // エラーなら２度目を試す
                    {                                       // 2度ダメならエラー処理
                        errMsg = "PICでの受信データが異常でした";
                        break;
                    }
                }
            }
            catch
            {   //例外が発生した時、USBが抜かれたとしてPortの切断処理 
                // 受信でタイムアウトの時もここでエラー
                btnClose_Click(this, null);
                errMsg = "送受信することができません";
            }
            // ------------------------------------------------------------------------
            // 送信の後処理　（エラーの表示とボタン表示切り替え）
            // ------------------------------------------------------------------------
            if (errMsg != "")  // エラーがあれば、その内容を表示し、復帰
            {              
                ShowMsg(errMsg);
                return;
            }
            // エラーが無ければ、Runボタンの色および文字を変える
            btnRun.BackColor = Color.Gold;
            if (rb3_FreeRun.Checked == false)
            {                              // FreeRun 以外
                btnRun.Text = "Cancel";
                isIdle = false;
                ShowMsg("トリガ待ち");
            }
            else
            {                               // FreeRun 
                btnRun.Text = "Stop";
                isIdle = false;
                pb_LED_ON.Visible =true;
            }
        }

        /***************************  freeCmd()関数 **********************************
         *  private void freeCmd()
         *  
         *  Description:
         *  FreeRun選択時はすぐ復帰
         *  その他の選択時は、
         *      トリガ待ちを中止するために、強制FreeRunコマンド「！」をPICに送信                
        ***************************************************************************/
        private void freeCmd()
        {
            if (rb3_FreeRun.Checked == true) return;  // FreeRunの時は非該当
            // FreeRun 以外を選択していれば ---------------------------------------
            try
            {   // 強制FreeRunコマンド「！」送信する
                serialPort1.Write("!");    
            }
            catch
            {   // 例外が発生した時、USBが抜かれたとしてPortの切断処理 
                // 受信でタイムアウトの時もここでエラー
                btnClose_Click(this, null);
                ShowMsg("送信することができません");
            }
        }

        /*************************** メッセージボタン *******************************
         *  private void bnMsg_Click(object sender, EventArgs e)
         *  
         *  Description:
         *      メッセージ（tbMsg）を表示/非表示する          
        ***************************************************************************/
        private void bnMsg_Click(object sender, EventArgs e)
        {
            if (bnMsg.Text == "表示")
            {
                bnMsg.Text = "非表示";
                tbMsg.Visible = true;
            }
            else
            {
                bnMsg.Text = "表示";
                tbMsg.Visible = false;
            }
        }

        /************************** メッセージ表示関数 ******************************
         *  private void ShowMsg(string msg)
         *  
         *  Description:
         *  引数がある場合、そのメッセージを表示にする
         *  引数が""の時は、メッセージ（tbMsg）を非表示にする
        ***************************************************************************/
        private void ShowMsg(string msg)
        {
            if (msg == "")
            {
                lbMsg.Enabled = false;
                bnMsg.Enabled = false;
                bnMsg.Text = "非表示";
                tbMsg.Visible = false;
            }
            else
            {
                lbMsg.Enabled = true;
                bnMsg.Enabled = true;
                bnMsg.Text = "非表示";
                tbMsg.Visible = true;
                tbMsg.Text = msg;
            }
        }

        /************************** 起動時 ******************************************
        *  private void Form1_Load(object sender, EventArgs e)
        *  
        *  Description:
        *  目盛を描く
        *  GND線を描く
       ***************************************************************************/
        private void Form1_Load(object sender, EventArgs e)
        {
            screenBack();
            // pictureBox2の親をpictureBox1とし、透明にする
            this.pictureBox2.Parent = this.pictureBox1;
            pictureBox2.BackColor = Color.Transparent;
            // カーソルラベルの親をpictureBox2とし、透明にする
            lbCrX.Parent = pictureBox2;
            lbCrY.Parent = pictureBox2;
        }

        /***************************  screenBack関数 ***********************************
          *  private void screenBack
          *  
          * Description:
          *    目盛を描く
          *    GND線を描く
          *    リソースを解放する
        ***************************************************************************/
        private void screenBack()
        {
            int i;
            // -----------------------------------------------------------------
            //描画先とするImageオブジェクトを作成する
            //ImageオブジェクトのGraphicsオブジェクトを作成する
            // -----------------------------------------------------------------
            Bitmap canvas = new Bitmap(pictureBox1.Width, pictureBox1.Height);
            Graphics g = Graphics.FromImage(canvas);
            // -----------------------------------------------------------------
            // 目盛を描く  canvasの幅と高さを基準に10pixの間隔
            //             副目盛は、上下に5pix高、主目盛は高さ一杯
            // -----------------------------------------------------------------
            for (i = 0; i < (canvas.Width / 10); i++)
            {
                if (i % 5 == 0)
                {   // 主目盛は、上下の貫通線
                    g.DrawLine(Pens.DarkGray, 10 * i, 0, 10 * i, canvas.Height);
                }
                else
                {   // 副目盛は、枠の上下に高さ5pixの線
                    g.DrawLine(Pens.DarkGray, 10 * i, 0, 10 * i, 5);
                    g.DrawLine(Pens.DarkGray, 10 * i, canvas.Height - 5, 10 * i, canvas.Height);
                }
            }

            // -----------------------------------------------------------------
            // データ線を描く  
            // -----------------------------------------------------------------
            int ch;        // 描いているチャネル
            // GND用太ペンの作成
            Pen boldPen = new Pen(Color.Lime, 2);
            if (cbDspBlack.Checked)
            {
                boldPen.Color = Color.Magenta;
            }
            // 各チャネルのLogic Lowの画描  ４Ch分繰り返す ---------------------
            for (ch = 1; ch < 5; ch++)
                g.DrawLine(boldPen, 0, ch * 30, canvas.Width, ch * 30);
            //リソースを解放する -----------------------------------------------
            boldPen.Dispose();
            //nomlPen.Dispose();
            g.Dispose();                //
            pictureBox1.Image = canvas; //PictureBox1に表示する
        }

        /***************************  マウスクリック *************************
         *  private void pictureBox2_MouseDown(object sender, MouseEventArgs e)
         *  
         *  Description:
         *  pictureBox2の上でマウスボタンがクリックされると、
         *      その位置を取得し、 pictureBox2の上の相対位置に変換
         *      ラベルに、位置を表示して線を引く
         *                     
        ************************************************************************/
        private void pictureBox2_MouseDown(object sender, MouseEventArgs e)
        {
            // マウスポインタの位置を画面座標で取得する
            // 得られた画面座標をクライアント座標に変換する
            System.Drawing.Point sp = System.Windows.Forms.Cursor.Position;
            System.Drawing.Point cp = this.PointToClient(sp);
            // Panel1 の位置とスクロール量を反映し、pictureBox2内の位置を計算
            int x = cp.X - 40 - panel1.AutoScrollPosition.X;
            // クリックしたボタンの左右で、保存場所を変える
            if (e.Button==MouseButtons.Left) Xpos = x;
            if (e.Button==MouseButtons.Right) Ypos = x;
            drawCoursor();           // カーソルを描画
            dspCursorPos();          // カーソル位置を表示
        }

        /***************************  drawCoursor関数 **************************
          *  private void drawCoursor()
          *  
          * Description:
          *    カーソルを描く
          *    リソースを解放する
        ***************************************************************************/
        private void drawCoursor()
        {
            // -----------------------------------------------------------------
            //描画先とするImageオブジェクトを作成する
            //ImageオブジェクトのGraphicsオブジェクトを作成する
            // -----------------------------------------------------------------
            Bitmap canvas = new Bitmap(pictureBox2.Width, pictureBox2.Height);
            Graphics g = Graphics.FromImage(canvas);
            // カーソルは、上下の貫通線
            g.DrawLine(Pens.CornflowerBlue, Xpos, 0, Xpos, canvas.Height);
            g.DrawLine(Pens.Violet, Ypos, 0, Ypos, canvas.Height);
            lbCrX.Location = new Point(Xpos-10, 30);
            lbCrY.Location = new Point(Ypos, 30);
            //リソースを解放する -----------------------------------------------
            g.Dispose();                //
            pictureBox2.Image = canvas; //PictureBox1に表示する
        }

        /*************************** カーソル位置表示関数 *********************
         *  private void cursorPosDsp()
         *  
         *  Description:
         *      Xカーソル、Yカーソルの時間的位置を表示する
         *      pictureBox2内の位置を示す共通変数：Xpos,Yposを使用する
         *      表示は、SampleRateを考慮した、時間単位に変換
                    //lbX.Text = string.Format("X = {0:d} {1}", x, unit);
                    //lbX.Text = "X = " + f.ToString("##.#") + " ms";        * 
        **********************************************************************/
        private void dspCursorPos()
        {
            int Tpos = 0;
            if (rb0_Start.Checked) Tpos = 100;   // トリガ位置をセット
            else if (rb1_Dly1Win.Checked) Tpos = -900;     // 
            else if (rb2_Dly2Win.Checked) Tpos = -1900;    // 
            else if (rb3_FreeRun.Checked) Tpos = 0;        // 
            else if (rb4_Center.Checked) Tpos = 550;       // 
            else if (rb5_End.Checked) Tpos = 1000;         // 
            int diff = Math.Abs(Ypos - Xpos);
            lbX.Text = "X = " + fmtTime(Xpos - Tpos);
            lbY.Text = "Y = " + fmtTime(Ypos - Tpos);
            lbYX.Text = "|Y - X|= " + fmtTime(diff) + fmtFreq(diff);
        }

        /***************************  時間表示変換 ************************
          * private string fmtTime(int pos)
          *  
          * Description:
          *    カーソル位置、サンプルレート、トリガ位置から
          *    カーソルの時間を計算する
          *    出力は、文字列
         *************************************************************************/
        private string fmtTime(int pos)
        {
            string str = "- - -";
            if (cbSampleRate.SelectedIndex == -1) return str;
            int[] RateValue = { 1, 2, 5, 10, 20, 50, 100, 200, 500, 1000 };
            int unit = RateValue[cbSampleRate.SelectedIndex];
            int Xtime = pos * unit;
            int AbsX = Math.Abs(Xtime);
            double dX = (double)Xtime / 1000.0;
            if (AbsX < 1000) str = string.Format("{0} us", Xtime);
            else if (AbsX < 10000) str = string.Format("{0:#.##} ms", dX);
            else if (AbsX < 100000) str = string.Format("{0:#.#} ms", dX);
            else str = string.Format("{0:#} ms", dX);
            return str;
        }

        /*************************** 周波数表示変換 ***************************
          * private string fmtTime(int pos)
          *  
          * Description:
          *    カーソル位置、サンプルレート、トリガ位置から
          *    カーソルの時間を計算する
          *    出力は、文字列
         ****************************************************************/
        private string fmtFreq(int pos)
        {
            string str = " ( - - Hz)";
            if (cbSampleRate.SelectedIndex == -1) return str;
            if (pos == 0) return str;
            int[] RateValue = { 1, 2, 5, 10, 20, 50, 100, 200, 500, 1000 };
            int unit = RateValue[cbSampleRate.SelectedIndex];
            double XFreq = 1000000.0 / (pos * unit);
            if (XFreq > 1000) str = string.Format(" ({0:#.#} kHz)", XFreq/1000.0);
            else str = string.Format(" ({0:#.#} Hz)", XFreq);
            return str;
        }

        /***************************  サンプルレート変更 **********************
          * private void cbSampleRate_SelectedIndexChanged(object sender, EventArgs e)
          *  
          * Description:
          *    サンプルレート変更したら、○○s / Div の値を更新する
          *    カーソルの時間間隔表示を更新する
        *********************************************************************/
        private void cbSampleRate_SelectedIndexChanged(object sender, EventArgs e)
        {
            string[] strDiv = { "50 us / Div", "100 us / Div", "250 us / Div",
                                "500 us / Div", "1 ms / Div", "2.5 ms / Div",
                                "5 ms / Div", "10 ms / Div", "25 ms / Div",
                                "50 ms / Div"};
            lbDiv.Text = strDiv[cbSampleRate.SelectedIndex];
            dspCursorPos();
        }

        /***************************  カーソル表示変更 ******************
         * private void checkBox1_CheckedChanged(object sender, EventArgs e)
         *  
         * Description:
         *    カーソル表示変更したら、カーソル関連表示をOn/Offする
         *    
       ***************************************************************************/
        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox1.Checked)  
            {
                lbX.Enabled = true;
                lbY.Enabled = true;
                lbYX.Enabled = true;
                pictureBox2.Visible = true;
            }
            else
            {
                lbX.Enabled = false;
                lbY.Enabled = false;
                lbYX.Enabled = false;
                pictureBox2.Visible = false;
            }
        }

        private void cbDspBlack_CheckedChanged(object sender, EventArgs e)
        {
            if (cbDspBlack.Checked)
            {
                pictureBox1.BackColor = Color.White;
            }
            else
            {
                pictureBox1.BackColor = Color.Black;
            }
            screenBack();   // Clear Display
        }
    }
}

